package com.mcwl.web.controller.resource;

import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.utils.obs.ObsUtils;
import com.mcwl.resource.service.impl.FileServiceImpl;
import com.mcwl.web.controller.common.OssUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 文件/图片
 *
 * @author DaiZibo
 * @date 2025/2/10
 * @apiNote
 */

@Slf4j
@Api(tags = "上传文件")
@RestController
@RequestMapping("/file")
public class FileController {

    @Autowired
    private FileServiceImpl fileService;

    @Autowired
    private ObsUtils obsUtils;

    /***
     *
     * 图片
     * @param file
     * @return
     */
    @ApiOperation("上传图片")
    @PostMapping("/imgUpload")
    public AjaxResult imgUpload(@RequestParam MultipartFile file) {

        String s = OssUtil.uploadMultipartFile(file);
        String fileName = file.getOriginalFilename();
        Map<String, String> map = new HashMap<>();
        map.put("fileName", fileName);
        map.put("url", s);
        return AjaxResult.success(map);
    }


    /***
     *
     * 上传文件
     * @param file
     * @return
     */
    @ApiOperation("上传文件")
    @PostMapping("/fileUpload")
    public AjaxResult fileUpload(@RequestParam MultipartFile file) {

        log.info("开始上传文件...");
        Map<String, String> map = obsUtils.uploadFile(file);

        return AjaxResult.success(map);
    }


    /***
     *
     * zip
     * @param file
     * @return
     */
    @ApiOperation(value = "zip")
    @PostMapping("/zipUrlFile")
    public AjaxResult zipUrlFile(@RequestParam MultipartFile file) {
        String s = OssUtil.uploadMultipartFile(file);
        return AjaxResult.success(s);
    }


    /***
     *
     * 下载zip
     * @param file
     * @return
     */
    @ApiOperation(value = "zip")
    @PostMapping("/zipUrl")
    public AjaxResult zipUrl(@RequestParam MultipartFile file) {
        String s = OssUtil.uploadMultipartFile(file);
        return AjaxResult.success(s);
    }


    /**
     * 根据文件名查找是否存在
     * @param name
     * @param type
     * @return
     */
    @GetMapping("/selectFileName")
    @ApiOperation(value = "根据文件名查找是否存在并返回秘钥")
    public AjaxResult selectFileName(@RequestParam String name,String type){

        return fileService.selectFileName(name,type);
    }

    @GetMapping("/selectFile")
    @ApiOperation(value = "根据文件名查找是否存在")
    public AjaxResult selectFile(@RequestParam String name,String type){

        return fileService.selectFile(name,type);
    }

    @GetMapping("/download")
    @ApiOperation(value = "读取文件内容")
    public AjaxResult download(@RequestParam String name) throws IOException {

        obsUtils.download(name);

        return AjaxResult.success();
    }

}
