package com.mcwl.common.config;


import org.springframework.stereotype.Component;

import java.util.BitSet;

/***
 * 布隆过滤器
 */

@Component
public class BloomFilterUtil {
    private static final int DEFAULT_SIZE = 2 << 24;    // 布隆过滤器的比特长度
    private static final int[] seeds = new int[] {7, 11, 13, 31,37, 61};    // 这里要选取质数，能很好的降低错误率

    private BitSet bits = new BitSet(DEFAULT_SIZE);
    private SimpleHash[] func = new SimpleHash[seeds.length];

    public static void main(String[] args) {
        String value = "crankzcool@gmail.com";
        BloomFilterUtil filter = new BloomFilterUtil();
//        System.out.println(filter.contains(value));
//        filter.add(value);
//        System.out.println(filter.contains(value));

        filter.add("1");
        filter.add("2");
        filter.remove("1");
        System.out.println(filter.contains("1"));
    }

    public void remove(String value) {
        if (value == null) {
            return;
        }
        for (SimpleHash f : func) {
            bits.set(f.hash(value), false);
        }
    }

    public BloomFilterUtil() {
        for (int i = 0; i < seeds.length; i++) {
            func[i] = new SimpleHash(DEFAULT_SIZE, seeds[i]);
        }
    }

    public void add(String value) {
        for (SimpleHash f: func) {
            bits.set(f.hash(value), true);
        }
    }

    public boolean contains(String value) {
        if (value == null) {
            return false;
        }
        boolean ret = true;
        for (SimpleHash f : func) {
            ret = ret && bits.get(f.hash(value));
        }
        return ret;
    }
    public static class SimpleHash {
        private int cap;
        private int seed;

        public SimpleHash(int cap, int seed) {
            this.cap = cap;
            this.seed = seed;
        }

        public int hash(String value) {
            int result = 0;
            int len = value.length();
            for (int i = 0; i < len; i++) {
                result = seed * result + value.charAt(i);
            }
            return (cap - 1) & result;
        }
    }


}
