package com.mcwl.memberCenter.domain.dto;

import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.mcwl.common.core.domain.BaseEntity;
import com.mcwl.memberCenter.enums.PromotionEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * 促销活动请求参数
 */
@Data
public class PromotionDto {

    @TableId
    private Long id;

    /**
     * 活动名称
     */
    @NotBlank(message = "活动名称不能为空")
    private String activityName;

    /**
     * 活动开始时间 校验时间格式
     */
    @NotNull(message = "活动开始时间不能为空")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;

    /**
     * 活动结束时间
     */
    @NotNull(message = "活动结束时间不能为空")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;

    /**
     * 活动类型，如“限时折扣”、“额外积分”
     */
    @NotNull()
    private PromotionEnum activityType;

    /**
     * 折扣/积分奖励 根据活动类型，这里可以是折扣率（如0.8代表8折）
     */
    @NotNull(message = "折扣/积分奖励不能为空")
    @Min(value = 0, message = "折扣/积分奖励不能小于0")
    private Double activityValue;

    /**
     * 活动的详细描述
     */
    private String description;

    /**
     * 适用会员等级 可选字段，用于指定哪些会员等级可以享受此活动（可以用逗号分隔的会员等级ID）
     */
    private String memberLevelIds;



}
