package com.mcwl.resource.domain;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.mcwl.common.core.domain.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * 模型表
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.common.domain.resource
 * @Filename：MallProduct
 * @Description 模型表
 * @Date：2024/12/30 17:22
 */
@AllArgsConstructor
@NoArgsConstructor
@Data
@Builder
@TableName("model")
@ApiModel(description = "模型表")
public class ModelProduct extends BaseEntity {

    /**
     * 主键ID
     */
    @ApiModelProperty(value = "主键ID")
    @TableId
    private Long id;
    /**
     * 名称
     */
    @ApiModelProperty(value = "名称")
    private String modelName;
    /***
     * 用户id
     */
    @ApiModelProperty(value = "用户id")
    private Long userId;
    /**
     * 模型类型
     */
    @ApiModelProperty(value = "模型类型")
    private Long modelType;
    /**
     * 垂类
     */
    @ApiModelProperty(value = "垂类")
    private String category;
    /**
     * 功能
     */
    @ApiModelProperty(value = "功能")
    private String functions;
    /**
     * 标签（最多三个，切割）
     */
    @ApiModelProperty(value = "标签（最多三个，切割）")
    private String tags;
    /**
     * 参与活动
     */
    @ApiModelProperty(value = "参与活动")
    private String activityId;
    /**
     * 是否原创
     */
    @ApiModelProperty(value = "是否原创")
    private Integer isOriginal;
    /**
     * 原创作者姓名
     */
    @ApiModelProperty(value = "原创作者姓名")
    private String originalAuthorName;
    /**
     * 生图次数
     */
    @ApiModelProperty(value = "生成图次数")
    private Integer reals;
    /**
     * 封面图
     */
    @ApiModelProperty(value = "封面图")
    private String surfaceUrl;
    /**
     * 下载次数
     */
    @ApiModelProperty(value = "下载次数")
    private Integer numbers;
    /**
     * 点赞数
     */
    @ApiModelProperty(value = "点赞数")
    private Integer likeNum;
    /**
     * 审核状态
     */
    @ApiModelProperty(value = "审核状态")
    private Integer auditStatus;

    /**
     * 审核（1公开 2自见）
     */
    @ApiModelProperty(value = "审核")
    private Integer jurisdiction;

    /**
     * 审核失败原因
     */
    @ApiModelProperty(value = "审核失败原因")
    private String auditText;
    /**
     * 是否推荐模型
     */
    @ApiModelProperty(value = "是否推荐模型")
    private Integer isRecommend;

    /**
     * 是否置顶
     */
    @ApiModelProperty(value = "是否置顶")
    private Integer isTop;


    /**
     * 删除标志（0代表存在 2代表删除）
     */
    @ApiModelProperty(value = "删除标志（0代表存在 2代表删除）")
    private String delFlag;

    /**
     * 商品价格
     */
    @ApiModelProperty(value = "商品价格")
    private Double productPrice;



    /**
     * 翻译后标签
     */
    @ApiModelProperty(value = "翻译后标签")
    @TableField(exist = false)
    private List<String> styleList;


    /**
     * 是否关注 0未关注 1关注
     */
    @ApiModelProperty(value = "是否关注 0未关注 1关注")
    @TableField(exist = false)
    private Integer isAttention;


    /**
     * 是否点赞 0未点赞 1点赞
     */
    @ApiModelProperty(value = "是否点赞 0未点赞 1点赞")
    @TableField(exist = false)
    private Integer isLike;

}
