package com.mcwl.resource.domain.dto;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * 工作流表
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.domain
 * @Filename：WorkFlow
 * @Description 工作流表
 * @Version：1.0
 * @Date：2025/1/8 19:38
 */
@Builder
@AllArgsConstructor
@NoArgsConstructor
@Data
@TableName("work_flow")
@ApiModel(description = "工作流表")
public class AddWorkFlow {

    /**
     * 主键ID
     */
    @ApiModelProperty(value = "主键ID")
    @TableId
    private Long id;
    /**
     * 用户id
     */
    @ApiModelProperty(value = "用户id")
    private Long userId;
    /**
     * 名称（最多30字）
     */
    @ApiModelProperty(value = "名称（最多30字）")
    @NotBlank(message = "名称不能为空")
    @Size(max = 30,message = "名称最多30字")
    private String workflowName;

    /**
     * 功能
     */
    @ApiModelProperty(value = "类别")
    private String type;
    /**
     * 参与活动
     */
    @ApiModelProperty(value = "参与活动")
    private String activityParticipation;
    /**
     * 删除标志（0代表存在 2代表删除）
     */
    @ApiModelProperty(value = "删除标志（0代表存在 2代表删除）")
    private Integer delFlag;

    /**
     * 审核状态（0全部状态 1已发布-公开 2已发布-仅自己可见 3审核中 4审核未通过）
     */
    @ApiModelProperty(value = "审核状态（0全部状态 1已发布-公开 2已发布-仅自己可见 3审核中 4审核未通过）")
    private Integer auditStatus = 3;

    /**
     *  审核失败原因
     */
    @ApiModelProperty(value = "审核失败原因")
    private String auditText;

    /**
     * 是否原创
     */
    @ApiModelProperty(value = "是否原创")
    @NotNull(message = "是否公开原创必选")
    private Integer original;

    /**
     * 原文作者名字
     */
    @ApiModelProperty(value = "原文作者名字")
    private String authorName;

    /**
     * 使用数量
     */
    @ApiModelProperty(value = "使用数量")
    private Long useNumber = 0L;

    /**
     *下载数量
     */
    @ApiModelProperty(value = "下载数量")
    private Long downloadNumber = 0L;

    /**
     * 是否允许在线使用(0允许 1不允许)
     */
    @ApiModelProperty(value = "是否允许在线使用(0允许 1不允许)")
    private Integer onlineUse;

    /**
     * 是否允许下载工作流(0允许 1不允许)
     */
    @ApiModelProperty(value = "是否允许下载工作流(0允许 1不允许)")
    private Integer download;

    /**
     * 是否允许出售或商用(0允许 1不允许
     */

    @ApiModelProperty(value = "是否允许出售或商用(0允许 1不允许)")
    private Integer sell;

    /**
     * 封面图地址
     */
    @ApiModelProperty(value = "封面图地址")
    private String coverPath;

    /**
     * 审核权限
     */
    @ApiModelProperty(value = "审核权限")
    @NotNull(message = "是否公开必选")
    private Integer jurisdiction;

    /**
     * 是否置顶
     */
    @ApiModelProperty(value = "是否置顶")
    private Integer isTop;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd",timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date createTime;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd",timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date updateTime;

    /**
     * 作品点赞数量
     */
    @ApiModelProperty(value = "作品点赞数量")
    private Integer likeCount = 0;


    /**
     * 翻译后类别
     */
    @ApiModelProperty(value = "翻译后类别")
    @TableField(exist = false)
    private List<String> typeList;

}
