package com.mcwl.resource.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.DictConstants;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.R;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.ModelVersion;
import com.mcwl.resource.mapper.ModelVersionMapper;
import com.mcwl.resource.service.ModelVersionService;
import com.mcwl.system.init.DictInit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**模型版本 业务实现层
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.service.impl
 * @Filename：ModelVersionServiceImpl
 * @Description TODO
 * @Date：2025/1/9 16:53
 */
@Service
public class ModelVersionServiceImpl  extends ServiceImpl< ModelVersionMapper,ModelVersion> implements ModelVersionService {

    @Autowired
    private ModelVersionService modelVersionService;


    @Autowired
    private ModelVersionMapper modelVersionMapper;


    @Override
    public List<ModelVersion> selectLogininforList(ModelVersion modelVersion) {

        // 创建一个 LambdaQueryWrapper 实例
        LambdaQueryWrapper<ModelVersion> queryWrapper = new LambdaQueryWrapper<>();

        // 使用非空检查来添加查询条件
        if (StringUtils.isNotNull(modelVersion.getIsExclusiveModel())) {
            queryWrapper.eq(ModelVersion::getIsExclusiveModel, modelVersion.getIsExclusiveModel());
        }


        if (StringUtils.isNotNull(modelVersion.getVersionName())) {
            queryWrapper.eq(ModelVersion::getVersionName, modelVersion.getVersionName());
        }

        return modelVersionMapper.selectList(queryWrapper);
    }

    @Override
    public R<List<ModelVersion>> selectByModelId(Long modelId) {

        //根据模型ID查询所有的版本信息
        LambdaQueryWrapper<ModelVersion> modelVersionLambdaQueryWrapper = new LambdaQueryWrapper<>();
        modelVersionLambdaQueryWrapper.eq(ModelVersion::getModelId,modelId);
        modelVersionLambdaQueryWrapper.eq(ModelVersion::getDelFlag,0);

        List<ModelVersion> modelVersions = modelVersionMapper.selectList(modelVersionLambdaQueryWrapper);
        for (ModelVersion modelVersion : modelVersions) {

            ArrayList<String> arrayList = new ArrayList<>();

            //获取
            String[] split = modelVersion.getHigh().split(",");

            for (String s : split) {
                arrayList.add(DictInit.getDictValue(DictConstants.DICT_TYPE_MODEL_VERSION_HIGH,s));
            }

            modelVersion.setHighList(arrayList);
        }
        return R.ok(modelVersions);
    }

    @Override
    public List<ModelVersion> finbyid(Long id) {

        LambdaQueryWrapper<ModelVersion> modelVersionLambdaQueryWrapper = new LambdaQueryWrapper<>();
        modelVersionLambdaQueryWrapper.eq(ModelVersion::getModelId,id);
        modelVersionLambdaQueryWrapper.eq(ModelVersion::getDelFlag,0);

        List<ModelVersion> modelVersions = baseMapper.selectList(modelVersionLambdaQueryWrapper);

        for (ModelVersion modelVersion : modelVersions) {

        }

        return modelVersions;
    }


}
