package com.mcwl.resource.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.R;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.Report;
import com.mcwl.resource.domain.vo.PageVo;
import com.mcwl.resource.mapper.ReportMapper;
import com.mcwl.resource.service.ReportService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * 举报 业务实现层
 *
 * @author DaiZibo
 * @date 2025/1/18
 * @apiNote
 */

@Service
public class ReportServiceImpl implements ReportService {


    @Autowired
    private ReportMapper reportMapper;

    @Override
    public void addReport(Report report) {

        report.setCreateTime(new Date());
        report.setUserId(SecurityUtils.getUserId());
        reportMapper.insert(report);
    }

    @Override
    public R<Page<Report>> selectReport(PageVo pageVo) {

        Page<Report> page = new Page<>(pageVo.getPageNumber(), pageVo.getPageSize());

        //构造查询条件
        LambdaQueryWrapper<Report> reportLambdaQueryWrapper = new LambdaQueryWrapper<>();
        reportLambdaQueryWrapper.eq(Report::getDelFlag, 0);
        if (pageVo.getType() != null) {
            reportLambdaQueryWrapper.eq(Report::getStatus, 0);
        }

        return R.ok(reportMapper.selectPage(page, reportLambdaQueryWrapper));
    }

    @Override
    public void deleteReport(Long id) {

        Report report = new Report().builder().id(id)
                .delFlag(2).build();

        reportMapper.updateById(report);
    }

    @Override
    public void updateStatus(Long id) {

        Report report = new Report().builder().id(id)
                .status(1)
                .build();

        reportMapper.updateById(report);
    }
}
